package cmd

import (
	"context"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/net/ghttp"
	"github.com/gogf/gf/v2/net/goai"
	"github.com/gogf/gf/v2/os/gcmd"
	"nodeMonitor/internal/consts"
	"nodeMonitor/internal/router"
	"nodeMonitor/internal/websocket"
)

var (
	Main = gcmd.Command{
		Name:  "main",
		Usage: "main",
		Brief: "start http server",
		Func: func(ctx context.Context, parser *gcmd.Parser) (err error) {

			//启动服务
			websocket.StartWebSocket(ctx)

			s := g.Server()

			s.Group("/", func(group *ghttp.RouterGroup) {

				router.BindController(group)
			})

			// Custom enhance API document.
			enhanceOpenAPIDoc(s)

			//err = StartPingStart(ctx)
			//if err != nil {
			//	return err
			//}
			//
			//defer func() {
			//
			//	gcron.Stop("ping_status")
			//	gcron.Remove("ping_status")
			//}()

			s.Run()
			return nil
		},
	}
)

func enhanceOpenAPIDoc(s *ghttp.Server) {
	openapi := s.GetOpenApi()
	openapi.Config.CommonResponse = ghttp.DefaultHandlerResponse{}
	openapi.Config.CommonResponseDataField = `Data`

	// API description.
	openapi.Info = goai.Info{
		Title:       consts.OpenAPITitle,
		Description: consts.OpenAPIDescription,
		Contact: &goai.Contact{
			Name: "GoFrame",
			URL:  "https://goframe.org",
		},
	}
}

//func test(ctx context.Context) {
//	taskStatusTime, err := g.Cfg().Get(ctx, "node.taskStatusTime")
//	if err != nil {
//		glog.Debug(ctx, err.Error())
//		return
//	}
//
//	s := fmt.Sprintf("0 */%d * * * *", taskStatusTime.Int())
//	t, err := task.GetCronNextTime(s, time.Now())
//	glog.Info(ctx, "NextTime:", t, err)
//
//	for i := 0; i < 3; i++ {
//		glog.Info(ctx, "执行任务:", i)
//	}
//}
//
//func TestCrom(ctx context.Context) error {
//	taskStatusTime, err := g.Cfg().Get(ctx, "node.taskStatusTime")
//	if err != nil {
//		glog.Debug(ctx, err.Error())
//		return err
//	}
//
//	taskName, err := g.Cfg().Get(ctx, "node.taskName")
//	if err != nil {
//		glog.Debug(ctx, err.Error())
//		return err
//	}
//	s := fmt.Sprintf("0 */%d * * * *", taskStatusTime.Int())
//	t, err := task.GetCronNextTime(s, time.Now())
//	glog.Info(ctx, "NextTime:", t, err)
//	_, err = gcron.AddSingleton(ctx, s, func(ctx context.Context) {
//		go test(ctx)
//	}, taskName.String())
//	return nil
//}
