package controller

import (
	"github.com/gogf/gf/v2/crypto/gmd5"
	"github.com/gogf/gf/v2/errors/gcode"
	"github.com/gogf/gf/v2/errors/gerror"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/net/ghttp"
	"github.com/gogf/gf/v2/os/glog"
	"golang.org/x/net/context"
	v1 "nodeMonitor/api/v1"
	"nodeMonitor/internal/base"
	"nodeMonitor/internal/model"
	"nodeMonitor/library/libUtils"
)

var Login = sLogin{}

type sLogin struct {
}

func (c *sLogin) Login(ctx context.Context, req *v1.LoginReq) (*v1.LoginRes, error) {
	res := new(v1.LoginRes)
	rootUsername, err := g.Cfg().Get(ctx, "node.rootUsername")
	if err != nil {
		glog.Debug(ctx, err.Error())
		return res, err
	}

	rootPassword, err := g.Cfg().Get(ctx, "node.rootPassword")
	if err != nil {
		glog.Debug(ctx, err.Error())
		return res, err
	}

	if req.UserName == rootUsername.String() && req.PassWord == rootPassword.String() {
		ip := libUtils.GetClientIp(ctx)
		userAgent := libUtils.GetUserAgent(ctx)
		key := gmd5.MustEncryptString(rootUsername.String()) + gmd5.MustEncryptString(rootPassword.String())
		if g.Cfg().MustGet(ctx, "gfToken.multiLogin").Bool() {
			key = gmd5.MustEncryptString(rootUsername.String()) + gmd5.MustEncryptString(rootPassword.String()+ip+userAgent)
		}
		token, err := base.GfToken().GenerateToken(ctx, key, model.LoginUserRes{
			UserUserName: rootUsername.String(),
			UserPassword: rootPassword.String(),
		})
		if err != nil {
			return res, err
		}
		res.Token = token
	} else {
		err = gerror.NewCode(gcode.New(401, "", nil), "账号或者密码错误")
		return res, err
	}

	return res, nil
}

func (c *sLogin) LoginOut(ctx context.Context, req *v1.LoginOutReq) (*v1.LoginOutRes, error) {
	res := new(v1.LoginOutRes)
	err := base.GfToken().RemoveToken(ctx, base.GfToken().GetRequestToken(ghttp.RequestFromCtx(ctx)))
	if err != nil {
		return res, err
	}
	return res, nil
}
