package controller

import (
	"github.com/gogf/gf/v2/errors/gcode"
	"github.com/gogf/gf/v2/errors/gerror"
	"golang.org/x/net/context"
	v1 "nodeMonitor/api/v1"
	"nodeMonitor/internal/model"
	"nodeMonitor/internal/service"
)

var NodeConfig = sNodeConfig{}

type sNodeConfig struct {
}

func (c *sNodeConfig) All(ctx context.Context, req *v1.NodeConfigAllReq) (res *v1.NodeConfigAllRes, err error) {
	res = new(v1.NodeConfigAllRes)
	res.NodeConfigItem, err = service.NodeConfig().GetAll(ctx)
	return
}

func (c *sNodeConfig) Add(ctx context.Context, req *v1.NodeConfigReq) (res *v1.NodeConfigRes, err error) {
	exist, err := service.NodeConfig().IsExist(ctx, req.ServerId)
	if err != nil {
		return
	}
	if !exist {
		err = gerror.NewCode(gcode.New(401, "", nil), "数据已经存在")
		return
	}

	err = service.NodeConfig().Create(ctx, model.NodeConfigAndEditCreateInput{
		ServerId: req.ServerId,
		SshCom:   req.SshCom,
		SshPass:  req.SshPass,
		SshUser:  req.SshUser,
	})

	return
}

func (c *sNodeConfig) Edit(ctx context.Context, req *v1.NodeConfigEditReq) (res *v1.NodeConfigEditRes, err error) {
	err = service.NodeConfig().Edit(ctx, model.NodeConfigAndEditCreateInput{
		ServerId: req.ServerId,
		SshCom:   req.SshCom,
		SshPass:  req.SshPass,
		SshUser:  req.SshUser,
	})
	return
}

func (c *sNodeConfig) Del(ctx context.Context, req *v1.NodeConfigDelReq) (res *v1.NodeConfigDelRes, err error) {
	err = service.NodeConfig().Delete(ctx, req.ServerId)
	return
}
