package middleware

import (
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/net/ghttp"
	"github.com/gogf/gf/v2/os/glog"
	"github.com/gogf/gf/v2/util/gconv"
	"nodeMonitor/internal/base"
	"nodeMonitor/internal/model"
	"nodeMonitor/library/libResponse"
)

type IMiddleware interface {
	CORS(r *ghttp.Request)
	Ctx(r *ghttp.Request)
	Auth(r *ghttp.Request)
}
type middlewareImpl struct{}

var middlewareService = middlewareImpl{}

func Middleware() IMiddleware {
	return IMiddleware(&middlewareImpl{})
}

func (s *middlewareImpl) CORS(r *ghttp.Request) {
	corsOptions := r.Response.DefaultCORSOptions()
	// you can set options
	//corsOptions.AllowDomain = []string{"goframe.org", "baidu.com"}
	r.Response.CORS(corsOptions)
	r.Middleware.Next()
}

// Ctx 自定义上下文对象
func (s *middlewareImpl) Ctx(r *ghttp.Request) {
	ctx := r.GetCtx()
	//url := gstr.TrimLeft(r.Request.URL.Path, "/")
	//if !gstr.Contains(url, "/api/v1/node/add") {
	//	// 执行下一步请求逻辑
	//	r.Middleware.Next()
	//}
	//r.Request.URL.Path
	// 初始化登录用户信息
	data, err := base.GfToken().ParseToken(r)
	if err != nil {
		// 执行下一步请求逻辑
		g.Log().Error(ctx, err)
		libResponse.FailJson(true, r, err.Error())
		r.Middleware.Next()
	}
	if data != nil {
		context := new(model.Context)
		err = gconv.Struct(data.Data, &context.User)
		if err != nil {
			g.Log().Error(ctx, err)
			// 执行下一步请求逻辑
			r.Middleware.Next()
		}
		base.Context().Init(r, context)
	}
	// 执行下一步请求逻辑
	r.Middleware.Next()
}

// Auth 权限判断处理中间件
func (s *middlewareImpl) Auth(r *ghttp.Request) {
	ctx := r.GetCtx()

	rootUsername, err := g.Cfg().Get(ctx, "node.rootUsername")
	if err != nil {
		glog.Debug(ctx, err.Error())
		r.Middleware.Next()
		return
	}

	//rootPassword, err := g.Cfg().Get(ctx, "node.rootPassword")
	//if err != nil {
	//	glog.Debug(ctx, err.Error())
	//	r.Middleware.Next()
	//	return
	//}

	//获取登陆用户id
	user := base.Context().GetLoginUser(ctx)
	if user.UserUserName != rootUsername.String() {
		libResponse.FailJson(true, r, "没有访问权限")
	}
	//accessParams := r.Get("accessParams").Strings()
	//accessParamsStr := ""
	//if len(accessParams) > 0 && accessParams[0] != "undefined" {
	//	accessParamsStr = "?" + gstr.Join(accessParams, "&")
	//}
	//url := gstr.TrimLeft(r.Request.URL.Path, "/") + accessParamsStr
	/*if r.Method != "GET" && adminId != 1 && !gstr.Contains(url, "api/v1/system/login") {
		libResponse.FailJson(true, r, "对不起！演示系统，不能删改数据！")
	}*/
	//获取无需验证权限的用户id
	//tagSuperAdmin := false
	//service.User().NotCheckAuthUserIds(ctx).Iterator(func(v interface{}) bool {
	//	if gconv.Uint64(v) == adminId {
	//		tagSuperAdmin = true
	//		return false
	//	}
	//	return true
	//})
	//if tagSuperAdmin {
	//	r.Middleware.Next()
	//	//不要再往后面执行
	//	return
	//}
	//获取地址对应的菜单id
	//menuList, err := service.AuthRule().GetIsMenuList(ctx)
	//if err != nil {
	//	g.Log().Error(ctx, err)
	//	libResponse.FailJson(true, r, "请求数据失败")
	//}
	//var menu *model.SysAuthRuleInfoRes
	//for _, m := range menuList {
	//	ms := gstr.SubStr(m.Name, 0, gstr.Pos(m.Name, "?"))
	//	if m.Name == url || ms == url {
	//		menu = m
	//		break
	//	}
	//}
	////只验证存在数据库中的规则
	//if menu != nil {
	//	//若存在不需要验证的条件则跳过
	//	if gstr.Equal(menu.Condition, "nocheck") {
	//		r.Middleware.Next()
	//		return
	//	}
	//	menuId := menu.Id
	//	//菜单没存数据库不验证权限
	//	if menuId != 0 {
	//		//判断权限操作
	//		enforcer, err := service.CasbinEnforcer(ctx)
	//		if err != nil {
	//			g.Log().Error(ctx, err)
	//			libResponse.FailJson(true, r, "获取权限失败")
	//		}
	//		groupPolicy := enforcer.GetFilteredGroupingPolicy(0,
	//			gconv.String(adminId))
	//		if len(groupPolicy) == 0 {
	//			libResponse.FailJson(true, r, "没有访问权限")
	//		}
	//		hasAccess := false
	//		for _, v := range groupPolicy {
	//			if enforcer.HasPolicy(v[1], gconv.String(menuId), "All") {
	//				hasAccess = true
	//				break
	//			}
	//		}
	//		if !hasAccess {
	//			libResponse.FailJson(true, r, "没有访问权限")
	//		}
	//	}
	//} else if menu == nil && accessParamsStr != "" {
	//	libResponse.FailJson(true, r, "没有访问权限")
	//}
	r.Middleware.Next()
}
