package router

import (
	"github.com/gogf/gf/v2/net/ghttp"
	"nodeMonitor/internal/controller"
	"nodeMonitor/internal/middleware"
	"nodeMonitor/internal/websocket"
)

func MiddlewareCORS(r *ghttp.Request) {

	corsOptions := r.Response.DefaultCORSOptions()
	//corsOptions.AllowDomain = []string{"goframe.org", "baidu.com"}
	r.Response.CORS(corsOptions)

	r.Response.CORSDefault()

	r.Middleware.Next()
}
func BindController(group *ghttp.RouterGroup) {
	group.Group("/api/v1", func(group *ghttp.RouterGroup) {

		group.Middleware(ghttp.MiddlewareHandlerResponse, MiddlewareCORS)
		NewWebsocket(group)
		NodeRouter(group)
		NodeConfigRouter(group)
		PingConfigRouter(group)
		LoginRouter(group)
		DomeRouter(group)
		TaskLogRouter(group)
	})

}

func NewWebsocket(group *ghttp.RouterGroup) {
	group.Group("/ws", func(group *ghttp.RouterGroup) {
		group.ALL("/socket", websocket.WsPage)
	})
}

func NodeRouter(group *ghttp.RouterGroup) {
	group.Group("/node", func(group *ghttp.RouterGroup) {
		group.Middleware(middleware.Middleware().Ctx)
		group.Middleware(middleware.Middleware().Auth)
		group.Bind(
			controller.Node,
		)
	})
}

func NodeConfigRouter(group *ghttp.RouterGroup) {
	group.Group("/node/config", func(group *ghttp.RouterGroup) {

		group.Bind(
			controller.NodeConfig,
		)
	})
}

func PingConfigRouter(group *ghttp.RouterGroup) {
	group.Group("/ping/config", func(group *ghttp.RouterGroup) {
		group.Bind(
			controller.PingConfig,
		)
	})
}

func LoginRouter(group *ghttp.RouterGroup) {
	group.Group("/auth", func(group *ghttp.RouterGroup) {

		group.Bind(
			controller.Login,
		)
	})
}

func DomeRouter(group *ghttp.RouterGroup) {
	group.Group("/hello", func(group *ghttp.RouterGroup) {
		group.Bind(
			controller.Hello,
		)
	})
}

func TaskLogRouter(group *ghttp.RouterGroup) {
	group.Group("/task/log", func(group *ghttp.RouterGroup) {

		group.Bind(
			controller.TaskLog,
		)
	})
}
